local awful = require "awful"
local vars = require "misc.vars"
local gears = require "gears"
local wibox = require "wibox"
local rubato = require "lib.rubato"
local cfg = require "misc.cfg"

local function button(color, color_focus, callback)
    local widget = wibox.widget {
        widget = wibox.container.background,
        forced_height = vars.button_size,
        forced_width = vars.button_size,
        bg = color,
        shape = vars.shape,
        buttons = { callback }
    }

    widget:connect_signal("mouse::enter", function()
        local timed = rubato.timed {
            duration = 0.1,
            intro = 0.05,
            pos = vars.button_size,
            subscribed = function(pos)
                widget.forced_width = pos
            end
        }

        timed.target = vars.button_size * 2
        widget.bg = color_focus
    end)

    widget:connect_signal("mouse::leave", function()
        local timed = rubato.timed {
            duration = 0.1,
            intro = 0.05,
            pos = vars.button_size * 2,
            subscribed = function(pos)
                widget.forced_width = pos
            end
        }

        timed.target = vars.button_size
        widget.bg = color
    end)

    return widget
end

client.connect_signal("request::titlebars", function(c)
    if not cfg.draw_titlebar then return end

    local titlebar =  awful.titlebar(c, { 
        position = "top",
        size = vars.button_size + vars.padding * 2
    })

    awful.titlebar.enable_tooltip = false

    titlebar.widget = {
        widget = wibox.container.margin,
        margins = vars.padding,
        {
            nil,
            nil,
            {
                button(vars.colors.green, vars.colors.bright.green, awful.button({}, 1, function()
                    c.maximized = not c.maximized
                end)),
                button(vars.colors.yellow, vars.colors.bright.yellow, awful.button({}, 1, function()
                    gears.timer.delayed_call(function()
                        c.minimized = true
                    end) 
                end)),
                button(vars.colors.red, vars.colors.bright.red, awful.button({}, 1, function()
                    c:kill()
                end)),

                spacing = vars.padding,
                layout = wibox.layout.fixed.horizontal
            },
            layout = wibox.layout.align.horizontal
        }
    }
end)
